@@ -110,6 +110,16 @@ class LinuxVirtual(Virtual):
                 virtual_facts['virtualization_role'] = 'guest'
                 found_virt = True
 
+        # If docker/containerd has a custom cgroup parent, checking /proc/1/cgroup (above) might fail.
+        # https://docs.docker.com/engine/reference/commandline/dockerd/#default-cgroup-parent
+        # Fallback to more rudimentary checks.
+        if os.path.exists('/.dockerenv') or os.path.exists('/.dockerinit'):
+            guest_tech.add('docker')
+            if not found_virt:
+                virtual_facts['virtualization_type'] = 'docker'
+                virtual_facts['virtualization_role'] = 'guest'
+                found_virt = True
+
         # ensure 'container' guest_tech is appropriately set
         if guest_tech.intersection(set(['docker', 'lxc', 'podman', 'openvz', 'containerd'])) or systemd_container:
             guest_tech.add('container')
